unit BeeGui_AddFrm;

{ Contains:

  beeGUI Add form.

  (C) 2005 Andrew Filinsky, Melchiorre Caruso

  Modifyed:

  v1.0.1 build 9155 - 2005/07/15 Melchiorre Caruso
  v1.0.1 build 9158 - 2005/07/24 Melchiorre Caruso

}

{$R-,Q-,S-}

interface

uses
  Spin,
  Math,
  Menus,
  ImgList,
  Buttons,
  Dialogs,
  Classes,
  SysUtils,
  Graphics,
  Controls,
  StdCtrls,
  ComCtrls,
  FileCtrl,
  Messages,
  ShellApi,

  BeeGui_Forms, BeeGui_ListView;

const
  MSG_SelectDir = 0;

type
  TAddFrm = class (TBGAddForm)
  
    AddFrm_Pages: TPageControl;
    AddFrm_Pages_General: TTabSheet;
    AddFrm_Pages_Files: TTabSheet;

    // Add Page
    AddFrm_MethodGB: TGroupBox;
    AddFrm_ActionGB: TGroupBox;
    ConfigFrm_PriorityGB: TGroupBox;
    AddFrm_DictionaryGB: TGroupBox;

    AddFrm_Dictionary: TComboBox;
    AddFrm_Method: TComboBox;
    AddFrm_Action: TComboBox;
    ConfigFrm_Priority: TComboBox;

    AddFrm_OptionsGB: TGroupBox;
    AddFrm_sOption: TCheckBox;
    AddFrm_tOption: TCheckBox;
    AddFrm_rOption: TCheckBox;
    AddFrm_pOption: TCheckBox;

    // Files page
    AddFrm_FilesTitle: TLabel;

    AddFrm_FilesImages: TImageList;

    AddFrm_PMenu: TPopupMenu;
    AddFrm_PMenu_AddDir: TMenuItem;
    AddFrm_PMenu_N2: TMenuItem;
    AddFrm_PMenu_AddFiles: TMenuItem;
    AddFrm_PMenu_N1: TMenuItem;
    AddFrm_PMenu_View: TMenuItem;
    AddFrm_PMenu_Type: TMenuItem;
    AddFrm_PMenu_Modify: TMenuItem;
    AddFrm_PMenu_Delete: TMenuItem;
    AddFrm_Root: TEdit;

    BtnOk: TBitBtn;
    BtnCancel: TBitBtn;
    AddFrm_BtnUpDown: TUpDown;

    // MSGs
    AddFrm_MSGs: TComboBox;

    // Dialog
    AddFrm_OpenDialog: TOpenDialog;
    AddFrm_cOption: TCheckBox;
    BtnDir: TSpeedButton;
    BtnFile: TSpeedButton;
    AddFrm_ForceExtGB: TGroupBox;
    AddFrm_eOption: TEdit;
    BtnView: TSpeedButton;
    BtnDelete: TSpeedButton;
    BtnModify: TSpeedButton;
    BtnType: TSpeedButton;
    AddFrm_Root_: TLabel;
    AddFrm_FilesList: TBGAddListView;

    procedure AddFrm_PMenu_AddDirClick (Sender: TObject);
    procedure AddFrm_PMenu_AddFilesClick (Sender: TObject);
    procedure AddFrm_PMenu_ViewClick (Sender: TObject);
    procedure AddFrm_PMenu_DeleteClick (Sender: TObject);
    procedure AddFrm_PMenu_TypeClick (Sender: TObject);
    procedure AddFrm_PMenu_ModifyClick (Sender: TObject);

    procedure FormCreate (Sender: TObject);
    procedure FormDestroy(Sender: TObject);

    procedure AddFrm_BtnUpDownClick (Sender: TObject; Button: TUDBtnType);
    procedure AddFrm_FilesListSelectItem (Sender: TObject; Item: TListItem; Selected: Boolean);
  private
    { Private declarations }
    
  end;

var
  AddFrm: TAddFrm;

implementation

uses
  BeeGui_PasswordFrm,
  BeeGui_Common,
  BeeGui_WinOS;

{$R *.DFM}

  procedure TAddFrm.AddFrm_PMenu_AddDirClick (Sender: TObject);
  var
    Dir: ansistring;
  begin
    if SelectDirectory (AddFrm_MSGs.Items [MSG_SelectDir], '', Dir) then
      AddFrm_FilesList.AddDirectory (Dir);
  end;

  procedure TAddFrm.AddFrm_PMenu_AddFilesClick (Sender: TObject);
  var
    I: integer;
  begin
    if AddFrm_OpenDialog.Execute then
      for I := 0 to AddFrm_OpenDialog.Files.Count - 1 do
        AddFrm_FilesList.AddFile (AddFrm_OpenDialog.Files [I]);
  end;

  procedure TAddFrm.AddFrm_PMenu_ViewClick (Sender: TObject);
  var
    I: integer;
  begin
    for I := 0 to AddFrm_FilesList.Items.Count -1 do
      if AddFrm_FilesList.Items [I].Selected then
        BG_ShellExecute (AddFrm_FilesList.Items [I].SubItems [0], '');
  end;
  
  procedure TAddFrm.AddFrm_PMenu_TypeClick (Sender: TObject);
  var
    I: integer;
  begin
    for I := AddFrm_FilesList.Items.Count - 1 downto 0 do
      if AddFrm_FilesList.Items [I].Selected then
      begin
        if AddFrm_FilesList.Items [I].ImageIndex = S_Index then
          AddFrm_FilesList.Items [I].ImageIndex := E_Index
        else
          AddFrm_FilesList.Items [I].ImageIndex := S_Index
      end;
    AddFrm_FilesList.UpdateItems;
  end;

  procedure TAddFrm.AddFrm_PMenu_ModifyClick (Sender: TObject);
  begin
    if not (AddFrm_FilesList.Selected = nil) then
      AddFrm_FilesList.Selected.EditCaption;
  end;

  procedure TAddFrm.AddFrm_BtnUpDownClick (Sender: TObject; Button: TUDBtnType);
  begin
    case Button of
      btPrev: if not (AnsiCompareText (AddFrm_FilesList.GetNewRoot (AddFrm_FilesList.Spin + 1), AddFrm_Root.Text) = 0) then
              begin
                AddFrm_FilesList.Spin := AddFrm_FilesList.Spin + 1;
                AddFrm_FilesList.UpdateItems;
              end;
              
      btNext: if not (AnsiCompareText (AddFrm_FilesList.GetNewRoot (AddFrm_FilesList.Spin - 1),  AddFrm_Root.Text) = 0) then
              begin
                AddFrm_FilesList.Spin := AddFrm_FilesList.Spin - 1;
                AddFrm_FilesList.UpdateItems;
              end;
    end;
  end;

  procedure TAddFrm.AddFrm_PMenu_DeleteClick (Sender: TObject);
  var
    I: integer;
  begin
    for I := AddFrm_FilesList.Items.Count - 1 downto 0 do
      if AddFrm_FilesList.Items [I].Selected then
        AddFrm_FilesList.Items.Delete (I);
    AddFrm_FilesList.UpdateItems;
  end;

  procedure TAddFrm.FormCreate(Sender: TObject);
  begin
    AddFrm_Pages.ActivePage := AddFrm_Pages_General;
    DragAcceptFiles (AddFrm_FilesList.Handle, True);
  end;

  procedure TAddFrm.FormDestroy(Sender: TObject);
  begin
    DragAcceptFiles (AddFrm_FilesList.Handle, False);
  end;

  procedure TAddFrm.AddFrm_FilesListSelectItem (Sender: TObject; Item: TListItem; Selected: Boolean);
  begin
    if AddFrm_FilesList.SelCount = 0 then
    begin
      AddFrm_PMenu_AddDir   .Enabled := True;
      AddFrm_PMenu_AddFiles .Enabled := True;
      // --
      AddFrm_PMenu_View     .Enabled := False;
      AddFrm_PMenu_Type     .Enabled := False;
      AddFrm_PMenu_Modify   .Enabled := False;
      // --
      AddFrm_PMenu_Delete   .Enabled := False;
    end else
    begin
      AddFrm_PMenu_AddDir   .Enabled := True;
      AddFrm_PMenu_AddFiles .Enabled := True;
      // --
      AddFrm_PMenu_View     .Enabled := True;
      AddFrm_PMenu_Type     .Enabled := True;
      AddFrm_PMenu_Modify   .Enabled := True;
      // --
      AddFrm_PMenu_Delete   .Enabled := True;
    end;

    BtnView  .Enabled := AddFrm_PMenu_View  .Enabled;
    BtnType  .Enabled := AddFrm_PMenu_Type  .Enabled;
    BtnModify.Enabled := AddFrm_PMenu_Modify.Enabled;
    BtnDelete.Enabled := AddFrm_PMenu_Delete.Enabled;
  end;



end.
