unit BeeGui_ListView;

{ Contains:

  TBGInfo  class
  TBGNode  class
  TBGList  class
  TBGLabel class

  TBGListView class
  TBGTreeView class
  TBGComboBox class
  
  TBGAddListView class

  (C) 2005 Andrew Filinsky, Melchiorre Caruso

  Modifyed:

  v1.0.1 build 9154 - 2005/07/14 Melchiorre Caruso
  v1.0.1 build 9160 - 2005/08/03 Melchiorre Caruso

}

{$R-,Q-,S-}


interface


uses
  Math,
  Classes,
  Dialogs,
  StdCtrls,
  Messages,
  SysUtils,
  Graphics,
  Controls,
  ComCtrls;

const
  S_Index = 0;
  E_Index = 1;

  MSG_SETDIR   = 'Loading...';
  TreeConst    = '\<Loading...>';


type

  TBGInfo = class
    ArcName: string;
    Version: string;
    FilesCount: integer;
    FilesSize: integer;
    FilesPacked: integer;
    FilesCrypted: integer;
    ArcSize: integer;
    Modified: integer;

    constructor Create;
    destructor Destroy; override;
  end;


type

  TBGNode = class
  public
    FileName: string;
    FilePath: string;
    FileType: string;
    Version: string;
    Encrypted: string;
    PackedSize: integer;
    Size: integer;
    Ratio: integer;
    Modified: TDateTime;
    Method: string;
    Attribute: integer;
    CRC: cardinal;
    Position: integer;
    ImageIndex: integer;
    
    destructor Destroy; override;
  end;


type

  TBGList = class (TList)
  public
    destructor Destroy; override;
    procedure  Clear; override;
    function   IndexOfDir (const dir: string): integer;
  end;


type

  TBGListView = class (TListView)
  private
    FCurrFiles: TList;
    FPathBox: TComboBox;
    FCurrPath: string;
    FFiles: TBGList;
    FDirs: TBGList;
    FColumnSort: integer;
    FListMode: boolean;
    FDirectionSort: boolean;
    FPathBoxSign: string;

  protected
    function    CompareFN (L: TList; Index1, Index2: integer): integer;
    procedure   QuickSort (List: TList; L, R: Integer);
    function    RescueDirectories: boolean;

    procedure   SetPath;

  private
    procedure   SetCurrPath (Value: string);
    procedure   SetPathBox (Value: TComboBox);
    procedure   SetPathBoxSign (Value: string);
    procedure   SetColumnSort (Value: integer);
    procedure   SetListMode (Value: boolean);

    // Event
    procedure   GetImageIndex (Sender: TObject; Item: TListItem);
    procedure   Data (Sender: TObject; Item: TListItem);

  public
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   Clear;

    function    Up: boolean;
    procedure   Initialize;

    property    Files: TBGList read FFiles write FFiles default nil;
    property    CurrFiles: TList read FCurrFiles write FCurrFiles default nil;
    property    CurrDir: string read FCurrPath write SetCurrPath;

  published
    property    PathBox: TComboBox read FPathBox write SetPathBox default nil;
    property    PathBoxSign: string read FPathBoxSign write SetPathBoxSign;
    property    ColumnSort: integer read FColumnSort write SetColumnSort;
    property    ListMode: boolean read FListMode write SetListMode default False;

  end;


type

  TBGTreeView = class (TTreeView)
  private
    FCurrPath: string;
    FPathEdit: TEdit;

    function    SetPath: boolean;
    
  private
    procedure   SetCurrPath (Value: string);
    procedure   SetEdit (Value: TEdit);

    //Event
    procedure   OnTreeExpanding (Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    procedure   OnTreeDblClick (Sender: TObject);
    procedure   OnTreeClick (Sender: TObject);
   
  public
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   Initialize;
    
  published
    property    CurrDir: string read FCurrPath write SetCurrPath;
    property    PathEdit: TEdit read FPathEdit write SetEdit default nil;

  end;


type

  TBGComboBox = class (TComboBox) end;


type

  TBGLabel = class (TLabel) end;

type

  TBGAddListView = class (TListView)
  private
    FRoot: TEdit;
    FSpin: integer;
    procedure SetRoot (Value: TEdit);

  protected
    function  ANDstring (const str1, str2: string): string;

  public
    procedure UpdateItems;
    function  GetNewRoot (NewSpin: integer): string;

    procedure AddFile (FileName: string);
    procedure AddDirectory (DirName: string);
    procedure AcceptFiles (var msg : TMessage); message WM_DROPFILES;

   published
    property  Root: TEdit read FRoot write SetRoot default nil;
    property  Spin: integer read FSpin write FSpin default 0;
  end;



procedure Register;



implementation

uses
  BeeGui_Common,
  BeeGui_WinOS,

  ShellAPI;



  // -------------------------------------------- //
  //                                              //
  // TBGInfo                                      //
  //                                              //
  // -------------------------------------------- //

  constructor TBGInfo.Create;
  begin
    SetLength (ArcName, 0);
  end;

  destructor  TBGInfo.Destroy;
  begin
    SetLength (ArcName, 0);
  end;



  // -------------------------------------------- //
  //                                              //
  // TBGNode                                      //
  //                                              //
  // -------------------------------------------- //

  destructor  TBGNode.Destroy;
  begin
    SetLength (FileName , 0);
    SetLength (FilePath , 0);
    SetLength (FileType , 0);
    SetLength (Version  , 0);
    SetLength (Encrypted, 0);
    SetLength (Method   , 0);
  end;



  // -------------------------------------------- //
  //                                              //
  // TBGList                                      //
  //                                              //
  // -------------------------------------------- //

  destructor  TBGList.Destroy;
  begin
    Clear;
    inherited Destroy;
  end;

  procedure   TBGList.Clear;
  var
    I: integer;
  begin
    for I := Count - 1 downto 0 do
      TBGNode (Items [I]).Destroy;
    inherited Clear;
  end;

  function    TBGList.IndexOfDir;
  var
    I: integer;
    Path, Name: string;
  begin
    Result := -1;

    Name := ExtractFileName (ExcludeDelimiter (dir));
    Path := ExtractFilePath (ExcludeDelimiter (dir));
    for I := 0 to Count - 1 do
      if CompareText (TBGNode (Items [I]).FilePath, Path) = 0 then
        if CompareText (TBGNode (Items [I]).FileName, Name) = 0 then
        begin
          Result := I;
          Break;
        end;
  end;



  // ------------------------------------------------------------- //
  //                                                               //
  // TBGListView                                                   //
  //                                                               //
  // ------------------------------------------------------------- //

  constructor TBGListView.Create;
  begin
    inherited Create (AOwner);
    Enabled         := False;
    FCurrPath       := '';
    OwnerData       := True;
    OwnerDraw       := False;

    FCurrFiles      := TList.Create;
    FFiles          := TBGList.Create;
    FDirs           := TBGList.Create;

    SmallImages     := TImageList.Create (nil);
    LargeImages     := TImageList.Create (nil);

    FColumnSort     := -1;
    FDirectionSort  := True;
    FListMode       := False;

    OnData          := Data;
    OnGetImageIndex := GetImageIndex;

    BG_LoadListViewImages (TImageList (SmallImages), TImageList (LargeImages));
  end;

  destructor  TBGListView.Destroy;
  begin
    SetLength (FCurrPath, 0);
    FFiles.Free;
    FDirs.Free;

    FCurrFiles.Free;
    SmallImages.Free;
    LargeImages.Free;
    inherited Destroy;
  end;

  function   TBGListView.CompareFN;
  var
    bool1, bool2 : boolean;
  begin
    with TBGNode (L.Items [Index1]) do bool1 := not ((faDirectory and Attribute) = faDirectory);
    with TBGNode (L.Items [Index2]) do bool2 := not ((faDirectory and Attribute) = faDirectory);

    if bool1 xor bool2 then
    begin
      if bool1 then
        Result := 1
      else
        Result := -1
    end else
    begin
      case FColumnSort of
         0: Result := CompareText (TBGNode (L.Items [Index1]).FileName   , TBGNode (L.Items [Index2]).FileName);
         1: Result := CompareText (TBGNode (L.Items [Index1]).FileType   , TBGNode (L.Items [Index2]).FileType);
         2: Result :=              TBGNode (L.Items [Index1]).PackedSize - TBGNode (L.Items [Index2]).PackedSize;
         3: Result :=              TBGNode (L.Items [Index1]).Size       - TBGNode (L.Items [Index2]).Size;
         4: Result :=              TBGNode (L.Items [Index1]).Ratio      - TBGNode (L.Items [Index2]).Ratio;
         5: Result := CompareText (TBGNode (L.Items [Index1]).Method     , TBGNode (L.Items [Index2]).Method);
         6: Result :=       Round (TBGNode (L.Items [Index1]).Modified   - TBGNode (L.Items [Index2]).Modified);
         7: Result :=              TBGNode (L.Items [Index1]).Attribute  - TBGNode (L.Items [Index2]).Attribute;
         8: Result :=              TBGNode (L.Items [Index1]).Attribute  - TBGNode (L.Items [Index2]).Attribute;
         9: Result := CompareText (TBGNode (L.Items [Index1]).FilePath   , TBGNode (L.Items [Index2]).FilePath);
        10: Result :=              TBGNode (L.Items [Index1]).Position   - TBGNode (L.Items [Index2]).Position;
        else
           Result := Index1 - Index2;
      end;
    end;

    if not FDirectionSort then
      if Result > 0 then
        Result := -1
      else
        if Result < 0 then
          Result := 1;
  end;

  procedure  TBGListView.QuickSort;
  var
    I,J, Pivot : Longint;
  begin
    Items.BeginUpdate;
    Items.Count := 0;

    if (R > L) then
    begin
      repeat
        I := L;
        J := R;
        Pivot := (L + R) div 2;
        repeat
          while CompareFn (List, I, Pivot) < 0 do Inc (I);
          while CompareFn (List, J, Pivot) > 0 do Dec (J);
          if I <= J then
          begin
            List.Exchange (I, J);
            if Pivot = I then
              Pivot := J
            else
              if Pivot = J then
                Pivot := I;
          Inc (I);
          Dec (j);
          end;
      until I > J;
      if L < J then QuickSort (List, L, J);
      L := I;
      until I >= R;
    end;

    Items.Count := FCurrFiles.Count;
    if not (FCurrFiles.Count = 0) then
      ItemFocused := Items [0];
    Items.EndUpdate;
  end;

  function   TBGListView.RescueDirectories;
  var
    I: integer;
    d: string;
    Node: TBGNode;
    Info: TBGFileInfo;
  begin
    FDirs.Clear;
    for I := 0 to FFiles.Count - 1 do
    begin
      d := TBGNode (FFiles.Items [I]).FilePath;
      while not (Length (d) = 0) do
      begin
        if FDirs.IndexOfDir (d) = -1 then
        begin
          Node            := TBGNode.Create;
          Node.FileName   := ExtractFileName (ExcludeDelimiter (d));
          Node.FilePath   := ExtractFilePath (ExcludeDelimiter (d));

          BG_GetFileInfo (Node.FileName, faDirectory, Info);

          Node.FileType   := Info.TypeName;
          Node.PackedSize := -1;
          Node.Size       := -1;
          Node.Ratio      := -1;
          Node.Modified   := Now;
          Node.Attribute  := faDirectory;
          Node.CRC        := Cardinal (-1);
          Node.Position   := -1;
          Node.ImageIndex := Info.IconIndex;

          FDirs.Add (Node);
        end;
        d := ExtractFilePath (ExtractFileDir (d));
      end;
    end;
    Result := not (FDirs.IndexOfDir (FCurrPath) = -1);
  end;

  procedure   TBGListView.SetPathBox;
  begin
    FPathBox := Value;
  end;

  procedure   TBGListView.SetPathBoxSign;
  begin
    FPathBoxSign := Value;
  end;

  procedure   TBGListView.SetColumnSort;
  begin
    if Enabled then
    begin
      if Value = FColumnSort then
        FDirectionSort := not FDirectionSort
      else
        FDirectionSort := True;
    end;

    FColumnSort := Value;
    QuickSort (FCurrFiles, 0, FCurrFiles.Count -1);
  end;

  procedure   TBGListView.SetListMode;
  begin
    FListMode := Value;
    SetPath;
  end;

  procedure   TBGListView.SetPath;
  var
    I, J: integer;
  begin
    FCurrFiles.Clear;
    if fListMode then
    begin
      for I := 0 to FFiles.Count -1 do
        FCurrFiles.Add (FFiles.Items [I]);
    end else
    begin
      for I := 0 to FDirs.Count -1 do
        if CompareText (FCurrPath , TBGNode (FDirs.Items [I]).FilePath) = 0 then
          FCurrFiles.Add (FDirs.Items [I]);

      for I := 0 to FFiles.Count -1 do
        if CompareText (FCurrPath , TBGNode (FFiles.Items [I]).FilePath) = 0 then
          FCurrFiles.Add (FFiles.Items [I]);

      if (not (Length (FPathBoxSign)  = 0)) and Assigned (FPathBox) then
      begin
        J := -1;
        for I := 0 to FPathBox.Items.Count -1 do
          if CompareText (FPathBoxSign + FCurrPath, FPathBox.Items [I]) = 0 then
          begin
            J := I;
            Break;
          end;

        if (J = -1) then
          FPathBox.ItemIndex := FPathBox.Items.Add (FPathBoxSign + FCurrPath)
        else
          FPathBox.ItemIndex := J;
      end;
    end;

    QuickSort (FCurrFiles, 0, FCurrFiles.Count -1);
  end;

  procedure   TBGListView.Initialize;
  begin
    if RescueDirectories  then
      SetCurrPath (FCurrPath)
    else
      SetCurrPath ('');
  end;

  procedure   TBGListView.Clear;
  begin
    SetLength (FCurrPath, 0);
    Items     .Clear;
    FFiles    .Clear;
    FDirs     .Clear;
    FCurrFiles.Clear;
  end;

  procedure   TBGListView.SetCurrPath;
  begin
    if Pos (FPathBoxSign, Value) = 1 then
      System.Delete (Value, 1, Length (FPathBoxSign));

    FCurrPath := IncludeDelimiter (Value);
    SetPath;
  end;

  function    TBGListView.Up;
  var
    d: string;
  begin
    Result := True;
    d := ExtractFilePath (ExtractFileDir (FCurrPath));
    if not (CompareText (FCurrPath, d) = 0) then
      SetCurrPath (d)
    else
      Result := False;  
  end;

  // Event ----------------------------------- //

  procedure   TBGListView.GetImageIndex;
  begin
    if Item = nil then Exit;
    Item.ImageIndex := TBGNode (FCurrFiles.Items [Item.Index]).ImageIndex;
  end;

  procedure   TBGListView.Data;
  begin
    if Item = nil then Exit;
    with TBGNode (FCurrFiles.Items [Item.Index]) do
    begin
      Item.Caption := FileName;
      Item.SubItems.Add (FileType);
      if (Attribute and faDirectory) = 0 then
      begin
        Item.SubItems.Add (SizeToStr (PackedSize));
        Item.SubItems.Add (SizeToStr (Size));
        Item.SubItems.Add (RatioToStr (Ratio));
        Item.SubItems.Add (Method);
        Item.SubItems.Add (DateTimeToStr (Modified));
        Item.SubItems.Add (AttrToStr (Attribute));
        Item.SubItems.Add (IntToStr (CRC));
        Item.SubItems.Add (FilePath);
        Item.SubItems.Add (IntToStr (Position));
      end else
      begin
        Item.SubItems.Add ('');
        Item.SubItems.Add ('');
        Item.SubItems.Add ('');
        Item.SubItems.Add ('');
        Item.SubItems.Add ('');
        Item.SubItems.Add (AttrToStr (Attribute));
        Item.SubItems.Add ('');
        Item.SubItems.Add (FilePath);
        Item.SubItems.Add ('');
      end;

    end;

  end;



  // ----------------------------------------------------- //
  //                                                       //
  // TBGTreeView                                           //
  //                                                       //
  // ----------------------------------------------------- //

  constructor TBGTreeView.Create;
  begin
    inherited Create (AOwner);
  end;

  destructor  TBGTreeView.Destroy;
  begin
    SetLength (FCurrPath, 0);
    inherited Destroy;
  end;

  procedure   TBGTreeView.Initialize;
  var
    I: integer;
    DriveList: TStringList;
    First, TMP: TTreeNode;
  begin
    FCurrPath := '';
    DriveList := TStringList.Create;
    Items.BeginUpdate;
    try
      Items.Clear;
      BG_GetDrivers (DriveList);
      First := Items.GetFirstNode;
      for I := 0 to DriveList.Count -1 do
      begin
        TMP := Items.Add (First, DriveList.Strings [I]);
        TMP.ImageIndex := BG_GetFileImageIndex (DriveList.Strings [I], 0);
        TMP.SelectedIndex := TMP.ImageIndex;

        TMP := Items.AddChild (TMP, treeconst);
        TMP.SelectedIndex := - 1;
        TMP.ImageIndex := - 1;
      end;
    finally
      Items.EndUpdate;
      DriveList.Free;
    end;

    OnExpanding := OnTreeExpanding;
    OnDblClick  := OnTreeDblClick;
    OnClick     := OnTreeClick;
  end;
  
  function    TBGTreeView.SetPath;
  var
    I: integer;
    cDir, tDir: string;
    Start: TTreeNode;
  begin
    Result := False;
    SetCurrentDir (FCurrPath);
    cDir := IncludeDelimiter (GetCurrentDir);

    I := 0;
    while I <  Items.Count do
    begin
      Start := Items [I];
      tDir  := IncludeDelimiter (Start.Text);
      if Start.Level > 0 then
        repeat
          Start := Start.Parent;
          tDir  := IncludeDelimiter (Start.Text) + tDir;
        until Start.Level = 0;

      if PosText (tDir, cDir) > 0 then
      begin
        Items [I].Expand (False);
        if CompareText (tDir, cDir) = 0 then
        begin
          Result := True;
          I := Items.Count
        end;
      end;
      Inc (I);
    end;
  end;

  procedure   TBGTreeView.SetCurrPath;
  begin
    FCurrPath := Value;
    SetPath;
  end;

  procedure   TBGTreeView.SetEdit;
  begin
    FPathEdit := Value;
  end;


  // Event --------------------------- //

  procedure   TBGTreeView.OnTreeExpanding;
  var
    Start, TMP: TTreeNode;
    DirList: TStringList;
    Path: string;
    I: integer;
  begin
    Start := Node;
    Selected := Node;

    if not Start.HasChildren then Exit;
    if not Start.GetFirstChild.ImageIndex = -1 then Exit;

    Path := IncludeDelimiter (Start.Text);
    if Start.Level > 0 then
      repeat
        Start := Start.Parent;
        Path := IncludeDelimiter (Start.Text) + Path;
      until Start.Level = 0;


    DirList := TStringList.Create;
    Items.BeginUpdate;
    try
      if BG_GetDirList (Path, DirList) then
        if Assigned (FPathEdit) then
          FPathEdit.Text := Path;

      Start := Node;
      while Start.HasChildren do Start.GetFirstChild.Delete;

      for I := 0 to DirList.Count - 1 do begin
        TMP := Items.AddChild (Start, DirList.Strings [I]);
        TMP.ImageIndex := BG_GetFileImageIndex (DirList.Strings [I], faDirectory);
        TMP.SelectedIndex :=  BG_GetFileOpenImageIndex (DirList.Strings [I], faDirectory);

        TMP := Items.AddChild (TMP, treeconst);
        TMP.SelectedIndex := - 1;
        TMP.ImageIndex := - 1;
      end;

    finally
      Items.EndUpdate;
      DirList.Free;
    end;
  end;

  procedure   TBGTreeView.OnTreeDblClick;
  begin
    if not (Selected = nil) then Selected.Expand (False);
  end;

  procedure   TBGTreeView.OnTreeClick;
  var
    Start: TTreeNode;
    Path: string;
  begin
    if not (Selected = nil) then
    begin
      Start := Selected;
      Path := IncludeDelimiter (Start.Text);
      if Start.Level > 0 then
        repeat
          Start := Start.Parent;
          Path := IncludeDelimiter (Start.Text) + Path;
        until Start.Level = 0;

      if SetCurrentDir (Path) then
        if Assigned (FPathEdit) then
          FPathEdit.Text := Path;
    end;
  end;



  // ----------------------------------------------------- //
  //                                                       //
  // TBGAddListView                                        //
  //                                                       //
  // ----------------------------------------------------- //

  //  fucntion and procedures

  function TBGAddListView.ANDstring;
  var
    J : integer;
  begin
    Result := '';
    for J := 1 to Min (Length (str1), Length (str2)) do
      if CompareText (str1 [J], str2 [J]) = 0 Then
        Result := Result + str1 [J]
      else
      begin
        if not (CompareText (IncludeDelimiter (Result), Result) = 0) Then
          Result := ExtractFilePath (Result);
        Exit;
      end;
  end;

  function TBGAddListView.GetNewRoot;
  var
    I: integer;
    S: TStringList;
  begin
    Result := '';
    S := TStringList.Create;
    try
      for I := 0 to Items.Count - 1 do
        if Items [I].ImageIndex = S_Index then
          S.Add (Items [I].SubItems [0] + Items [I].Caption);

      if not (S.Count = 0) then
      begin
        Result := ExtractFilePath (S.Strings [0]);
        for I := 1 to S.Count -1 do
          Result := ANDstring (Result, S.Strings [I]);
      end;

      for I := 1 to NewSpin do Result := ExtractFilePath (ExcludeDelimiter (Result));
    finally
      S.Free;
    end;
  end;

  procedure TBGAddListView.UpdateItems;
  var
    I, DirPos: integer;
    R, F: string;
  begin
    R := GetNewRoot (Spin);
    for I := Items.Count -1 downto 0 do
    begin
      F := Items [I].SubItems [0] + Items [I].Caption;

      DirPos := Pos (R, F);
      if (
           (DirPos > 0) or (Items [I].ImageIndex = S_Index)
         )
         or
         (
           (Items [I].ImageIndex = E_Index) and ( (DirPos > 0) or ((R = '') and (SelCount = 0)) )
         )
         then
        begin
          System.Delete (F, DirPos, Length (R));
          Items [I].Caption := F;
          Items [I].SubItems [0] := R;
        end;
    end;

    if Assigned (FRoot) then FRoot.Text := R;
  end;

  procedure TBGAddListView.AddFile;
  var
    lvItem: TListItem;
  begin
    Items.BeginUpdate;
    lvItem := Items.Add;
    with lvItem do
    begin
      Caption := ExtractFileName (FileName);
      SubItems.Add (ExtractFilePath (FileName));
      ImageIndex := S_Index;
    end;
    Items.EndUpdate;
    UpdateItems;
  end;

  procedure TBGAddListView.AddDirectory;
  var
    Name: string;
    lvItem: TListItem;
  begin
    Items.BeginUpdate;
    Name := IncludeDelimiter (ExtractFileName (DirName));
    lvItem := Items.Add;
    with lvItem do
    begin
      Caption := Name + '*.*';
      SubItems.Add (ExtractFilePath (DirName));
      ImageIndex := S_Index;
    end;
    Items.EndUpdate;
    UpdateItems;
  end;

  procedure TBGAddListView.AcceptFiles;
  const
    cnMaxFileNameLen = 255;
  var
    I, nCount: integer;
    acFileName: array [0..cnMaxFileNameLen] of char;
  begin
    nCount := DragQueryFile (msg.WParam, $FFFFFFFF, acFileName, cnMaxFileNameLen);
    for I := 0 to nCount - 1 do
    begin
      DragQueryFile (msg.WParam, i, acFileName, cnMaxFileNameLen);
      if FileExists (acFileName) then
         AddFile (acFileName)
      else
         AddDirectory (acFileName);
    end;
    DragFinish (msg.WParam);
  end;

  procedure TBGAddListView.SetRoot;
  begin
    FRoot := Value;
  end;



  // ----------------------------------------------------- //
  //                                                       //
  // Register                                              //
  //                                                       //
  // ----------------------------------------------------- //

  procedure   Register;
  begin
    RegisterComponents ('BeeGUI', [TBGListView   ]);
    RegisterComponents ('BeeGUI', [TBGTreeView   ]);
    RegisterComponents ('BeeGUI', [TBGComboBox   ]);
    RegisterComponents ('BeeGUI', [TBGLabel      ]);
    RegisterComponents ('BeeGUI', [TBGAddListView]);
  end;

end.
